#!/usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright (C) 2021  Paul Wolneykien <manowar@altlinux.org>
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import cups
import os
import pwd
import sys
import re
import tempfile
from datetime import date
from gi.repository import Gtk
import pluggy

pluginimpl = pluggy.HookimplMarker("system_config_printer")

@pluginimpl
def add_actions (host, job_action_group):
        fonarik = Fonarik (host)
        job_action_group.add_actions ([
                ("mark-job", None, u"Маркировка задания", None, None,
                 fonarik.on_job_mark_activate)
                ])

@pluginimpl
def adjust_ui (job_ui_manager):
        job_ui_manager.add_ui_from_string (
"""
<ui>
 <accelerator action="mark-job"/>
</ui>
"""
        )

@pluginimpl
def adjust_context_menu (job_action_group, job_context_menu):
        action = job_action_group.get_action ("mark-job")
        action.set_sensitive (False)
        item = action.create_menu_item ()
        item.show ()
        job_context_menu.append (item)

@pluginimpl
def adjust_toolbar (job_action_group, toolbar):
        action = job_action_group.get_action ("mark-job")
        action.set_sensitive (True)
        item = action.create_tool_item ()
        item.show ()
        toolbar.insert (item, -1)

class Fonarik:
        def __init__ (self, host):
                self.host = host

        def mark ( self, jobID, title, copies, addrs, phone, place ):
                conn = cups.Connection()
        
                attr = conn.getJobAttributes( jobID )
        
                tmpd, tmppath = tempfile.mkstemp()
                fonarik = os.fdopen( tmpd, "w")
                skip = 56-addrs.count("\n")
                try:
                    dt1 =  pwd.getpwnam(attr['job-originating-user-name'])
                except KeyError:
                    dt1 = [attr['job-originating-user-name'], 0, 0, 0, 0]
                dt2 =  pwd.getpwuid(os.getuid())
                rmargin = "\n       "
                fonarikText =  ("\n" * skip) + '       '+title+"\n       "+u'Экземпляров '+repr(copies)+ rmargin + place + rmargin +addrs + rmargin +u'Исполнитель '+(dt1[4] if dt1[4] else dt1[0])+ rmargin +u'Тел. '+phone+ rmargin +u'Отпечатал '+(dt2[4] if dt2[4] else dt2[0])+ rmargin +date.today().strftime("%d.%m.%Y")+"\n"
                fonarik.write( fonarikText )
                fonarik.flush()
        
                queue = re.split( '/', attr['job-printer-uri'])[-1]
                for i in range( int(copies) ) :
                        copy = repr(i+1)
                        newJobID = conn.createJob( queue, title+' '+u'Экз.№'+copy, {'alt-copy-number': copy.encode('utf-8'), 'alt-index': title.encode('utf-8'), 'job-hold-until': 'indefinite'})
                        conn.altDocFromToJob( queue , newJobID, jobID, 1)
        
                        newJob = conn.printFile( queue, tmppath , u'Фонарик: '+title+u' Экз.№'+copy, {'job-hold-until': 'indefinite','alt-copy-number': copy.encode('utf-8'), 'alt-index': title.encode('utf-8')})
        
                fonarik.close()
                os.remove( tmppath )
                conn.cancelJob( jobID, True )

        def on_job_mark_ok(self, data):
                for jobid in self.host.jobids:
                    buf = self.addresses.get_buffer()
                    startb, endb = buf.get_bounds()
                    self.mark (jobid,
                               self.docIndex.get_text(),
                               int(self.copiesNumber.get_value()),
                               buf.get_text(startb, endb, True),
                               self.docPhone.get_text(),
                               self.docPlace.get_text()
                    )
                self.MarkWindow.hide()
                self.MarkWindow.destroy()

        def on_job_mark_cancel(self, data):
                self.MarkWindow.hide()
                self.MarkWindow.destroy()

        def on_job_mark_activate(self, menuitem):
                self.MarkWindow = Gtk.Window()
                self.MarkWindow.set_title (_("Атрибуты задания"))
                self.MarkWindow.set_position(Gtk.WindowPosition.MOUSE)
                self.MarkWindow.set_transient_for (self.host.JobsWindow)
         
                table = Gtk.Table( 2, 6, False )
                self.MarkWindow.add( table )
                label = Gtk.Label(label=u"Количество копий:")
                table.attach ( label, 0, 1, 0, 1)
                label.show()
                copiesNumberAdj = Gtk.Adjustment(1, 1, 100, 1, 10, 0)
         #      self.copiesNumber = Gtk.SpinButton()
         #      self.copiesNumber.set_adjustment(copiesNumberAdj)
                self.copiesNumber = Gtk.SpinButton.new(copiesNumberAdj, 0.0, digits=0)
                table.attach (self.copiesNumber , 1, 2, 0, 1)
                self.copiesNumber.show()
         
                label = Gtk.Label(label=u"Инвентарный номер:")
                table.attach ( label, 0, 1, 1, 2)
                label.show()
                self.docIndex = Gtk.Entry()
                table.attach ( self.docIndex, 1, 2, 1, 2)
                self.docIndex.show()
         
                label = Gtk.Label(label=u"Адреса рассылки:")
                table.attach ( label, 0, 1, 2, 3)
                label.show()
                self.addresses = Gtk.TextView()
                table.attach ( self.addresses, 1, 2, 2, 3)
                self.addresses.show()
         
                label = Gtk.Label(label=u"Телефон исполнителя:")
                table.attach ( label, 0, 1, 3, 4)
                label.show()
                self.docPhone = Gtk.Entry()
                table.attach ( self.docPhone, 1, 2, 3, 4)
                self.docPhone.show()
         
                label = Gtk.Label(label=u"Идентификатор рабочего места:")
                table.attach ( label, 0, 1, 4, 5)
                label.show()
                self.docPlace = Gtk.Entry()
                table.attach ( self.docPlace, 1, 2, 4, 5)
                self.docPlace.show()
         
                butOk = Gtk.Button(stock=Gtk.STOCK_OK)
                butOk.connect("clicked", self.on_job_mark_ok)
                table.attach ( butOk, 1, 2, 5, 6)
                butOk.show()
         
                butCancel = Gtk.Button(stock=Gtk.STOCK_CANCEL)
                butCancel.connect("clicked", self.on_job_mark_cancel)
                table.attach ( butCancel, 0, 1, 5, 6)
                butCancel.show()
         
                table.show()
         
                self.MarkWindow.present()
